style

package
v0.1.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jun 19, 2024 License: MIT Imports: 3 Imported by: 0

Documentation

Overview

Package style provides helper functions for styling plot.Figure cells

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Color

func Color(color color.RGBA) plot.Style

Color creates a new cell style with the given color

func FontSize

func FontSize(size int) plot.Style

FontSize creates a new cell style with the given font size

func Name

func Name(name string) plot.Style

Name creates a new cell style with the given name

Types

This section is empty.

Jump to